from dataclasses import dataclass

from dataclasses_json import dataclass_json

from snooker.models import JsonModel


@dataclass_json
@dataclass
class Round(JsonModel):
    Round: int
    RoundName: str
    EventID: int
    MainEvent: int
    Distance: int
    NumLeft: int
    NumMatches: int
    Note: str
    ValueType: str
    Rank: int
    Money: float
    SeedGetsHalf: int
    ActualMoney: float
    Currency: str
    ConversionRate: int
    Points: int
    SeedPoints: int



import logging
from abc import abstractmethod
from typing import Any, List, Optional, Type, Union  # noqa

import requests

from snooker.models import JsonModel

ReturnType = Optional[Union[Type[JsonModel], List[Type[JsonModel]]]]


class BaseApi:
    def __init__(self, headers: dict = dict()):
        self.session = requests.Session()

        self.session.headers.update(**headers)

    @property
    @abstractmethod
    def base_url(self) -> str:
        pass

    @staticmethod
    def _unmarshal(data: dict, model: Type[JsonModel]) -> Any:
        try:
            return model.schema().load(data, many=True)
        except Exception:
            logging.warning('Error unmarshaling data.', extra=dict(data=data, model=model), exc_info=True)

            return None

    def _get_one_dimensional(self, params: dict, model: Type[JsonModel]) -> Any:
        result = None

        data = self._make_request(params)

        if data:
            result = BaseApi._unmarshal(data, model)

        if not result:
            return None
        else:
            return result[0]

    def _get_two_dimensional(self, params: dict, model: Type[JsonModel]) -> Any:
        result = None

        data = self._make_request(params)

        if data:
            result = BaseApi._unmarshal(data, model)

        if not result:
            return None
        else:
            return result

    def _make_request(self, params: dict = dict()) -> Optional[dict]:
        try:
            response = self.session.get(self.base_url, params=params)
        except Exception as e:
            raise e

        try:
            return response.json()
        except Exception:
            logging.warning('Error retrieving json data.', extra=dict(params=params), exc_info=True)

            return None



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from snooker.models import JsonModel


@dataclass_json
@dataclass
class Event(JsonModel):
    ID: int
    Name: str
    StartDate: str
    EndDate: str
    Sponsor: str
    Season: int
    Type: str
    Num: int
    Venue: str
    City: str
    Country: str
    Discipline: str
    Main: int
    Sex: str
    AgeGroup: str
    Url: str
    Related: str
    Stage: str
    ValueType: str
    ShortName: str
    WorldSnookerId: int
    RankingType: str
    EventPredictionID: int
    Team: bool
    Format: int
    Twitter: str
    HashTag: str
    ConversionRate: int
    AllRoundsAdded: bool
    PhotoURLs: str
    NumCompetitors: int
    NumUpcoming: int
    NumActive: int
    NumResults: int
    Note: str
    CommonNote: str
    DefendingChampion: int
    PreviousEdition: int
    Tour: str



from typing import List, Optional

from typing_extensions import Literal

from snooker.api.base import BaseApi
from snooker.models.snooker_org.event import Event
from snooker.models.snooker_org.match import Match
from snooker.models.snooker_org.player import Player
from snooker.models.snooker_org.ranking import Ranking
from snooker.models.snooker_org.round import Round
from snooker.models.snooker_org.seeding import Seeding

PlayerStatus = Literal['a', 'p']


class SnookerOrgApi(BaseApi):
    """
    Class serving as interface for snooker.org API.
    """

    @property
    def base_url(self) -> str:
        """
        Parameter describing snooker.org API URL.

        :return: snooker.org API URL
        """
        return 'http://api.snooker.org'

    def event(self, event_id: int) -> Optional[Event]:
        """
        Retrieve info regarding specific event.

        :param event_id: ID of an event
        :return: Event info (if available)
        """
        model = Event
        params = {'e': event_id}

        return self._get_one_dimensional(params, model)

    def match(self, event_id: int, round_id: int, match_no: int) -> Optional[Match]:
        """
        Retrieve info regarding specific match.

        :param event_id: Event within which match took place
        :param round_id: Event round id
        :param match_no: Match number
        :return: Match info (if available)
        """
        model = Match
        params = {'e': event_id, 'r': round_id, 'n': match_no}

        return self._get_one_dimensional(params, model)

    def player(self, player_id: int) -> Optional[Player]:
        """
        Retrieve info regarding a player.

        :param player_id: ID of a player
        :return: Player info (if available)
        """
        model = Player
        params = {'p': player_id}

        return self._get_one_dimensional(params, model)

    def season_events(self, season: int) -> Optional[List[Event]]:
        """
        Retrieve list of all events within a season.

        :param season: ID of a season. It is a year the given season has started. Use -1 for all seasons.
        :return: List of events (if available)
        """
        params = {'t': 5, 's': season}
        model = Event

        return self._get_two_dimensional(params, model)

    def event_matches(self, event_id: int) -> Optional[List[Match]]:
        """
        Retrieve list of matches in the event.

        :param event_id: ID of the event
        :return: List of matches (if available)
        """
        params = {'t': 6, 'e': event_id}
        model = Match

        return self._get_two_dimensional(params, model)

    def ongoing_matches(self) -> Optional[List[Match]]:
        """
        Retrieve ongoing matches. It returns also matches that are currently on break.

        :return: List of ongoing matches (if available)
        """
        params = {'t': 7}
        model = Match

        return self._get_two_dimensional(params, model)

    def player_matches(self, player_id: int, season: int) -> Optional[List[Match]]:
        """
        Retrieve list of all player matches within particular season.

        :param player_id: ID of a player
        :param season: ID of a season. Use -1 value to list player matches from all seasons
        :return: List of matches (if available)
        """
        params = {'t': 8, 'p': player_id, 's': season}
        model = Match

        return self._get_two_dimensional(params, model)

    def event_players(self, event_id: int) -> Optional[List[Player]]:
        """
        Retrieve list of all players participating in an event.

        :param event_id: ID of an event
        :return: List of players (if available)
        """
        params = {'t': 9, 'e': event_id}
        model = Player

        return self._get_two_dimensional(params, model)

    def players(self, status: PlayerStatus, season: Optional[int]) -> Optional[List[Player]]:
        """
        Retrieve list of all players in the tour in given season.

        :param status: Status of a player. Use a for amateur and p for professional
        :param season: Season for which data is collected. Use -1 for all seasons
        :return: List of players (if available)
        """
        params = {'t': 10, 'st': status, 's': season}
        model = Player

        return self._get_two_dimensional(params, model)

    def rankings(self, ranking_type: str, season: int) -> Optional[List[Ranking]]:
        """
        Retrieve rankings for particular season.

        :param ranking_type: Type of ranking. Available ranking types: http://api.snooker.org/help.html#RankingTypes
        :param season: ID of a season for which data will be collected
        :return: List of rankings (if available)
        """
        params = {'rt': ranking_type, 's': season}
        model = Ranking

        return self._get_two_dimensional(params, model)

    def round_info_by_event(self, event_id: int) -> Optional[List[Round]]:
        """
        Retrieve round info for particular event.

        :param event_id: ID of an event
        :return: List of rounds (if available)
        """
        params = {'t': 12, 'e': event_id}
        model = Round

        return self._get_two_dimensional(params, model)

    def round_info_by_season(self, season: Optional[int]) -> Optional[List[Round]]:
        """
        Retrieve round info for particular season.

        :param season: ID of a season for which data will be collected (current season if no season provided)
        :return: List of rounds (if available)
        """
        params = {'t': 12, 's': season}
        model = Round

        return self._get_two_dimensional(params, model)

    def event_seeding(self, event_id: int) -> Optional[List[Seeding]]:
        """
        Retrieve info regarding player seedings for an event.

        :param event_id: ID of an event
        :return: List of player seedings (if available)
        """
        params = {'t': 13, 'e': event_id}
        model = Seeding

        return self._get_two_dimensional(params, model)

    def upcoming_matches(self) -> Optional[List[Match]]:
        """
        Retrieve upcoming matches.

        :return: List of upcoming matches (if available)
        """
        params = {'t': 14}
        model = Match

        return self._get_two_dimensional(params, model)

    def results_from_last_days(self, number_of_days: int = 0) -> Optional[List[Match]]:
        """
        Retrieve results from the last few days. There is a limit of 500 results returned.

        :param number_of_days: number of last days to retrieve results for
        :return: List of matches (If available)
        """
        params = {'t': 15, 'ds': number_of_days}
        model = Match

        return self._get_two_dimensional(params, model)

    def head_to_head(self, player_1_id: int, player_2_id: int, season: Optional[int]) -> Optional[List[Match]]:
        """
        Retrieve info regarding a player.

        :param player_1_id: ID of a first player
        :param player_2_id: ID of a second player
        :param season: Season for which data is collected. Use -1 for all seasons (current if no season provided).
        :return: List of matches (if available)
        """
        params = {'p1': player_1_id, 'p2': player_2_id, 's': season}
        model = Match

        return self._get_two_dimensional(params, model)



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from snooker.models import JsonModel


@dataclass_json
@dataclass
class Player(JsonModel):
    ID: int
    Type: int
    FirstName: str
    MiddleName: str
    LastName: str
    TeamName: str
    TeamNumber: int
    TeamSeason: int
    ShortName: str
    Nationality: str
    Sex: str
    BioPage: str
    Born: str
    Twitter: str
    SurnameFirst: bool
    License: str
    Club: str
    URL: str
    Photo: str
    PhotoSource: str
    FirstSeasonAsPro: int
    LastSeasonAsPro: int
    Info: str
    NumRankingTitles: int
    NumMaximums: int
    Died: str



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from snooker.models import JsonModel


@dataclass_json
@dataclass
class Seeding(JsonModel):
    EventID: int
    PlayerID: int
    Seeding: int



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from snooker.models import JsonModel


@dataclass_json
@dataclass
class Match(JsonModel):
    ID: int
    EventID: int
    Round: int
    Number: int
    Player1ID: int
    Score1: int
    Walkover1: bool
    Player2ID: int
    Score2: int
    Walkover2: bool
    WinnerID: int
    Unfinished: bool
    OnBreak: bool
    Status: int
    WorldSnookerID: int
    LiveUrl: str
    DetailsUrl: str
    PointsDropped: bool
    ShowCommonNote: bool
    Estimated: bool
    Type: int
    TableNo: int
    VideoURL: str
    InitDate: str
    ModDate: str
    StartDate: str
    EndDate: str
    ScheduledDate: str
    FrameScores: str
    Sessions: str
    Note: str
    ExtendedNote: str
    HeldOver: bool
    StatsURL: str



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from snooker.models import JsonModel


@dataclass_json
@dataclass
class Ranking(JsonModel):
    ID: int
    Position: int
    PlayerID: int
    Season: int
    Sum: float
    Type: str



import os

from setuptools import find_packages, setup

__version__ = '0.6.0'

requirements_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'requirements.txt')
readme_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'README.md')

with open(requirements_path) as requirements_file:
    requirements = requirements_file.readlines()

with open(readme_path) as readme_file:
    long_description = readme_file.read()

setup(
    name='snooker',
    version=__version__,
    description='Snooker Data API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://www.github.com/mgorsk1/snooker',
    maintainer='mgorsk1',
    maintainer_email='gorskimariusz13@gmail.com',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    zip_safe=False,
    install_requires=requirements,
    python_requires='>=3.6',
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
